/*

 Package: dyncall
 Library: test
 File: test/suite3/case.h
 Description: 
 License:

   Copyright (c) 2011-2018 Daniel Adler <dadler@uni-goettingen.de>,
                           Tassilo Philipp <tphilipp@potion-studios.com>

   Permission to use, copy, modify, and distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
   copyright notice and this permission notice appear in all copies.

   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
   WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
   ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

/* auto generated by mkcase.py (on stdout) */
VF0(0,f_)
VF1(1,DCint,f_i)
VF1(2,DClonglong,f_L)
VF1(3,DCdouble,f_d)
VF1(4,DCfloat,f_f)
VF2(5,DCint,DCint,f_ii)
VF2(6,DClonglong,DCint,f_Li)
VF2(7,DCdouble,DCint,f_di)
VF2(8,DCfloat,DCint,f_fi)
VF2(9,DCint,DClonglong,f_iL)
VF2(10,DClonglong,DClonglong,f_LL)
VF2(11,DCdouble,DClonglong,f_dL)
VF2(12,DCfloat,DClonglong,f_fL)
VF2(13,DCint,DCdouble,f_id)
VF2(14,DClonglong,DCdouble,f_Ld)
VF2(15,DCdouble,DCdouble,f_dd)
VF2(16,DCfloat,DCdouble,f_fd)
VF2(17,DCint,DCfloat,f_if)
VF2(18,DClonglong,DCfloat,f_Lf)
VF2(19,DCdouble,DCfloat,f_df)
VF2(20,DCfloat,DCfloat,f_ff)
VF3(21,DCint,DCint,DCint,f_iii)
VF3(22,DClonglong,DCint,DCint,f_Lii)
VF3(23,DCdouble,DCint,DCint,f_dii)
VF3(24,DCfloat,DCint,DCint,f_fii)
VF3(25,DCint,DClonglong,DCint,f_iLi)
VF3(26,DClonglong,DClonglong,DCint,f_LLi)
VF3(27,DCdouble,DClonglong,DCint,f_dLi)
VF3(28,DCfloat,DClonglong,DCint,f_fLi)
VF3(29,DCint,DCdouble,DCint,f_idi)
VF3(30,DClonglong,DCdouble,DCint,f_Ldi)
VF3(31,DCdouble,DCdouble,DCint,f_ddi)
VF3(32,DCfloat,DCdouble,DCint,f_fdi)
VF3(33,DCint,DCfloat,DCint,f_ifi)
VF3(34,DClonglong,DCfloat,DCint,f_Lfi)
VF3(35,DCdouble,DCfloat,DCint,f_dfi)
VF3(36,DCfloat,DCfloat,DCint,f_ffi)
VF3(37,DCint,DCint,DClonglong,f_iiL)
VF3(38,DClonglong,DCint,DClonglong,f_LiL)
VF3(39,DCdouble,DCint,DClonglong,f_diL)
VF3(40,DCfloat,DCint,DClonglong,f_fiL)
VF3(41,DCint,DClonglong,DClonglong,f_iLL)
VF3(42,DClonglong,DClonglong,DClonglong,f_LLL)
VF3(43,DCdouble,DClonglong,DClonglong,f_dLL)
VF3(44,DCfloat,DClonglong,DClonglong,f_fLL)
VF3(45,DCint,DCdouble,DClonglong,f_idL)
VF3(46,DClonglong,DCdouble,DClonglong,f_LdL)
VF3(47,DCdouble,DCdouble,DClonglong,f_ddL)
VF3(48,DCfloat,DCdouble,DClonglong,f_fdL)
VF3(49,DCint,DCfloat,DClonglong,f_ifL)
VF3(50,DClonglong,DCfloat,DClonglong,f_LfL)
VF3(51,DCdouble,DCfloat,DClonglong,f_dfL)
VF3(52,DCfloat,DCfloat,DClonglong,f_ffL)
VF3(53,DCint,DCint,DCdouble,f_iid)
VF3(54,DClonglong,DCint,DCdouble,f_Lid)
VF3(55,DCdouble,DCint,DCdouble,f_did)
VF3(56,DCfloat,DCint,DCdouble,f_fid)
VF3(57,DCint,DClonglong,DCdouble,f_iLd)
VF3(58,DClonglong,DClonglong,DCdouble,f_LLd)
VF3(59,DCdouble,DClonglong,DCdouble,f_dLd)
VF3(60,DCfloat,DClonglong,DCdouble,f_fLd)
VF3(61,DCint,DCdouble,DCdouble,f_idd)
VF3(62,DClonglong,DCdouble,DCdouble,f_Ldd)
VF3(63,DCdouble,DCdouble,DCdouble,f_ddd)
VF3(64,DCfloat,DCdouble,DCdouble,f_fdd)
VF3(65,DCint,DCfloat,DCdouble,f_ifd)
VF3(66,DClonglong,DCfloat,DCdouble,f_Lfd)
VF3(67,DCdouble,DCfloat,DCdouble,f_dfd)
VF3(68,DCfloat,DCfloat,DCdouble,f_ffd)
VF3(69,DCint,DCint,DCfloat,f_iif)
VF3(70,DClonglong,DCint,DCfloat,f_Lif)
VF3(71,DCdouble,DCint,DCfloat,f_dif)
VF3(72,DCfloat,DCint,DCfloat,f_fif)
VF3(73,DCint,DClonglong,DCfloat,f_iLf)
VF3(74,DClonglong,DClonglong,DCfloat,f_LLf)
VF3(75,DCdouble,DClonglong,DCfloat,f_dLf)
VF3(76,DCfloat,DClonglong,DCfloat,f_fLf)
VF3(77,DCint,DCdouble,DCfloat,f_idf)
VF3(78,DClonglong,DCdouble,DCfloat,f_Ldf)
VF3(79,DCdouble,DCdouble,DCfloat,f_ddf)
VF3(80,DCfloat,DCdouble,DCfloat,f_fdf)
VF3(81,DCint,DCfloat,DCfloat,f_iff)
VF3(82,DClonglong,DCfloat,DCfloat,f_Lff)
VF3(83,DCdouble,DCfloat,DCfloat,f_dff)
VF3(84,DCfloat,DCfloat,DCfloat,f_fff)
VF4(85,DCint,DCint,DCint,DCint,f_iiii)
VF4(86,DClonglong,DCint,DCint,DCint,f_Liii)
VF4(87,DCdouble,DCint,DCint,DCint,f_diii)
VF4(88,DCfloat,DCint,DCint,DCint,f_fiii)
VF4(89,DCint,DClonglong,DCint,DCint,f_iLii)
VF4(90,DClonglong,DClonglong,DCint,DCint,f_LLii)
VF4(91,DCdouble,DClonglong,DCint,DCint,f_dLii)
VF4(92,DCfloat,DClonglong,DCint,DCint,f_fLii)
VF4(93,DCint,DCdouble,DCint,DCint,f_idii)
VF4(94,DClonglong,DCdouble,DCint,DCint,f_Ldii)
VF4(95,DCdouble,DCdouble,DCint,DCint,f_ddii)
VF4(96,DCfloat,DCdouble,DCint,DCint,f_fdii)
VF4(97,DCint,DCfloat,DCint,DCint,f_ifii)
VF4(98,DClonglong,DCfloat,DCint,DCint,f_Lfii)
VF4(99,DCdouble,DCfloat,DCint,DCint,f_dfii)
VF4(100,DCfloat,DCfloat,DCint,DCint,f_ffii)
VF4(101,DCint,DCint,DClonglong,DCint,f_iiLi)
VF4(102,DClonglong,DCint,DClonglong,DCint,f_LiLi)
VF4(103,DCdouble,DCint,DClonglong,DCint,f_diLi)
VF4(104,DCfloat,DCint,DClonglong,DCint,f_fiLi)
VF4(105,DCint,DClonglong,DClonglong,DCint,f_iLLi)
VF4(106,DClonglong,DClonglong,DClonglong,DCint,f_LLLi)
VF4(107,DCdouble,DClonglong,DClonglong,DCint,f_dLLi)
VF4(108,DCfloat,DClonglong,DClonglong,DCint,f_fLLi)
VF4(109,DCint,DCdouble,DClonglong,DCint,f_idLi)
VF4(110,DClonglong,DCdouble,DClonglong,DCint,f_LdLi)
VF4(111,DCdouble,DCdouble,DClonglong,DCint,f_ddLi)
VF4(112,DCfloat,DCdouble,DClonglong,DCint,f_fdLi)
VF4(113,DCint,DCfloat,DClonglong,DCint,f_ifLi)
VF4(114,DClonglong,DCfloat,DClonglong,DCint,f_LfLi)
VF4(115,DCdouble,DCfloat,DClonglong,DCint,f_dfLi)
VF4(116,DCfloat,DCfloat,DClonglong,DCint,f_ffLi)
VF4(117,DCint,DCint,DCdouble,DCint,f_iidi)
VF4(118,DClonglong,DCint,DCdouble,DCint,f_Lidi)
VF4(119,DCdouble,DCint,DCdouble,DCint,f_didi)
VF4(120,DCfloat,DCint,DCdouble,DCint,f_fidi)
VF4(121,DCint,DClonglong,DCdouble,DCint,f_iLdi)
VF4(122,DClonglong,DClonglong,DCdouble,DCint,f_LLdi)
VF4(123,DCdouble,DClonglong,DCdouble,DCint,f_dLdi)
VF4(124,DCfloat,DClonglong,DCdouble,DCint,f_fLdi)
VF4(125,DCint,DCdouble,DCdouble,DCint,f_iddi)
VF4(126,DClonglong,DCdouble,DCdouble,DCint,f_Lddi)
VF4(127,DCdouble,DCdouble,DCdouble,DCint,f_dddi)
VF4(128,DCfloat,DCdouble,DCdouble,DCint,f_fddi)
VF4(129,DCint,DCfloat,DCdouble,DCint,f_ifdi)
VF4(130,DClonglong,DCfloat,DCdouble,DCint,f_Lfdi)
VF4(131,DCdouble,DCfloat,DCdouble,DCint,f_dfdi)
VF4(132,DCfloat,DCfloat,DCdouble,DCint,f_ffdi)
VF4(133,DCint,DCint,DCfloat,DCint,f_iifi)
VF4(134,DClonglong,DCint,DCfloat,DCint,f_Lifi)
VF4(135,DCdouble,DCint,DCfloat,DCint,f_difi)
VF4(136,DCfloat,DCint,DCfloat,DCint,f_fifi)
VF4(137,DCint,DClonglong,DCfloat,DCint,f_iLfi)
VF4(138,DClonglong,DClonglong,DCfloat,DCint,f_LLfi)
VF4(139,DCdouble,DClonglong,DCfloat,DCint,f_dLfi)
VF4(140,DCfloat,DClonglong,DCfloat,DCint,f_fLfi)
VF4(141,DCint,DCdouble,DCfloat,DCint,f_idfi)
VF4(142,DClonglong,DCdouble,DCfloat,DCint,f_Ldfi)
VF4(143,DCdouble,DCdouble,DCfloat,DCint,f_ddfi)
VF4(144,DCfloat,DCdouble,DCfloat,DCint,f_fdfi)
VF4(145,DCint,DCfloat,DCfloat,DCint,f_iffi)
VF4(146,DClonglong,DCfloat,DCfloat,DCint,f_Lffi)
VF4(147,DCdouble,DCfloat,DCfloat,DCint,f_dffi)
VF4(148,DCfloat,DCfloat,DCfloat,DCint,f_fffi)
VF4(149,DCint,DCint,DCint,DClonglong,f_iiiL)
VF4(150,DClonglong,DCint,DCint,DClonglong,f_LiiL)
VF4(151,DCdouble,DCint,DCint,DClonglong,f_diiL)
VF4(152,DCfloat,DCint,DCint,DClonglong,f_fiiL)
VF4(153,DCint,DClonglong,DCint,DClonglong,f_iLiL)
VF4(154,DClonglong,DClonglong,DCint,DClonglong,f_LLiL)
VF4(155,DCdouble,DClonglong,DCint,DClonglong,f_dLiL)
VF4(156,DCfloat,DClonglong,DCint,DClonglong,f_fLiL)
VF4(157,DCint,DCdouble,DCint,DClonglong,f_idiL)
VF4(158,DClonglong,DCdouble,DCint,DClonglong,f_LdiL)
VF4(159,DCdouble,DCdouble,DCint,DClonglong,f_ddiL)
VF4(160,DCfloat,DCdouble,DCint,DClonglong,f_fdiL)
VF4(161,DCint,DCfloat,DCint,DClonglong,f_ifiL)
VF4(162,DClonglong,DCfloat,DCint,DClonglong,f_LfiL)
VF4(163,DCdouble,DCfloat,DCint,DClonglong,f_dfiL)
VF4(164,DCfloat,DCfloat,DCint,DClonglong,f_ffiL)
VF4(165,DCint,DCint,DClonglong,DClonglong,f_iiLL)
VF4(166,DClonglong,DCint,DClonglong,DClonglong,f_LiLL)
VF4(167,DCdouble,DCint,DClonglong,DClonglong,f_diLL)
VF4(168,DCfloat,DCint,DClonglong,DClonglong,f_fiLL)
VF4(169,DCint,DClonglong,DClonglong,DClonglong,f_iLLL)
VF4(170,DClonglong,DClonglong,DClonglong,DClonglong,f_LLLL)
VF4(171,DCdouble,DClonglong,DClonglong,DClonglong,f_dLLL)
VF4(172,DCfloat,DClonglong,DClonglong,DClonglong,f_fLLL)
VF4(173,DCint,DCdouble,DClonglong,DClonglong,f_idLL)
VF4(174,DClonglong,DCdouble,DClonglong,DClonglong,f_LdLL)
VF4(175,DCdouble,DCdouble,DClonglong,DClonglong,f_ddLL)
VF4(176,DCfloat,DCdouble,DClonglong,DClonglong,f_fdLL)
VF4(177,DCint,DCfloat,DClonglong,DClonglong,f_ifLL)
VF4(178,DClonglong,DCfloat,DClonglong,DClonglong,f_LfLL)
VF4(179,DCdouble,DCfloat,DClonglong,DClonglong,f_dfLL)
VF4(180,DCfloat,DCfloat,DClonglong,DClonglong,f_ffLL)
VF4(181,DCint,DCint,DCdouble,DClonglong,f_iidL)
VF4(182,DClonglong,DCint,DCdouble,DClonglong,f_LidL)
VF4(183,DCdouble,DCint,DCdouble,DClonglong,f_didL)
VF4(184,DCfloat,DCint,DCdouble,DClonglong,f_fidL)
VF4(185,DCint,DClonglong,DCdouble,DClonglong,f_iLdL)
VF4(186,DClonglong,DClonglong,DCdouble,DClonglong,f_LLdL)
VF4(187,DCdouble,DClonglong,DCdouble,DClonglong,f_dLdL)
VF4(188,DCfloat,DClonglong,DCdouble,DClonglong,f_fLdL)
VF4(189,DCint,DCdouble,DCdouble,DClonglong,f_iddL)
VF4(190,DClonglong,DCdouble,DCdouble,DClonglong,f_LddL)
VF4(191,DCdouble,DCdouble,DCdouble,DClonglong,f_dddL)
VF4(192,DCfloat,DCdouble,DCdouble,DClonglong,f_fddL)
VF4(193,DCint,DCfloat,DCdouble,DClonglong,f_ifdL)
VF4(194,DClonglong,DCfloat,DCdouble,DClonglong,f_LfdL)
VF4(195,DCdouble,DCfloat,DCdouble,DClonglong,f_dfdL)
VF4(196,DCfloat,DCfloat,DCdouble,DClonglong,f_ffdL)
VF4(197,DCint,DCint,DCfloat,DClonglong,f_iifL)
VF4(198,DClonglong,DCint,DCfloat,DClonglong,f_LifL)
VF4(199,DCdouble,DCint,DCfloat,DClonglong,f_difL)
VF4(200,DCfloat,DCint,DCfloat,DClonglong,f_fifL)
VF4(201,DCint,DClonglong,DCfloat,DClonglong,f_iLfL)
VF4(202,DClonglong,DClonglong,DCfloat,DClonglong,f_LLfL)
VF4(203,DCdouble,DClonglong,DCfloat,DClonglong,f_dLfL)
VF4(204,DCfloat,DClonglong,DCfloat,DClonglong,f_fLfL)
VF4(205,DCint,DCdouble,DCfloat,DClonglong,f_idfL)
VF4(206,DClonglong,DCdouble,DCfloat,DClonglong,f_LdfL)
VF4(207,DCdouble,DCdouble,DCfloat,DClonglong,f_ddfL)
VF4(208,DCfloat,DCdouble,DCfloat,DClonglong,f_fdfL)
VF4(209,DCint,DCfloat,DCfloat,DClonglong,f_iffL)
VF4(210,DClonglong,DCfloat,DCfloat,DClonglong,f_LffL)
VF4(211,DCdouble,DCfloat,DCfloat,DClonglong,f_dffL)
VF4(212,DCfloat,DCfloat,DCfloat,DClonglong,f_fffL)
VF4(213,DCint,DCint,DCint,DCdouble,f_iiid)
VF4(214,DClonglong,DCint,DCint,DCdouble,f_Liid)
VF4(215,DCdouble,DCint,DCint,DCdouble,f_diid)
VF4(216,DCfloat,DCint,DCint,DCdouble,f_fiid)
VF4(217,DCint,DClonglong,DCint,DCdouble,f_iLid)
VF4(218,DClonglong,DClonglong,DCint,DCdouble,f_LLid)
VF4(219,DCdouble,DClonglong,DCint,DCdouble,f_dLid)
VF4(220,DCfloat,DClonglong,DCint,DCdouble,f_fLid)
VF4(221,DCint,DCdouble,DCint,DCdouble,f_idid)
VF4(222,DClonglong,DCdouble,DCint,DCdouble,f_Ldid)
VF4(223,DCdouble,DCdouble,DCint,DCdouble,f_ddid)
VF4(224,DCfloat,DCdouble,DCint,DCdouble,f_fdid)
VF4(225,DCint,DCfloat,DCint,DCdouble,f_ifid)
VF4(226,DClonglong,DCfloat,DCint,DCdouble,f_Lfid)
VF4(227,DCdouble,DCfloat,DCint,DCdouble,f_dfid)
VF4(228,DCfloat,DCfloat,DCint,DCdouble,f_ffid)
VF4(229,DCint,DCint,DClonglong,DCdouble,f_iiLd)
VF4(230,DClonglong,DCint,DClonglong,DCdouble,f_LiLd)
VF4(231,DCdouble,DCint,DClonglong,DCdouble,f_diLd)
VF4(232,DCfloat,DCint,DClonglong,DCdouble,f_fiLd)
VF4(233,DCint,DClonglong,DClonglong,DCdouble,f_iLLd)
VF4(234,DClonglong,DClonglong,DClonglong,DCdouble,f_LLLd)
VF4(235,DCdouble,DClonglong,DClonglong,DCdouble,f_dLLd)
VF4(236,DCfloat,DClonglong,DClonglong,DCdouble,f_fLLd)
VF4(237,DCint,DCdouble,DClonglong,DCdouble,f_idLd)
VF4(238,DClonglong,DCdouble,DClonglong,DCdouble,f_LdLd)
VF4(239,DCdouble,DCdouble,DClonglong,DCdouble,f_ddLd)
VF4(240,DCfloat,DCdouble,DClonglong,DCdouble,f_fdLd)
VF4(241,DCint,DCfloat,DClonglong,DCdouble,f_ifLd)
VF4(242,DClonglong,DCfloat,DClonglong,DCdouble,f_LfLd)
VF4(243,DCdouble,DCfloat,DClonglong,DCdouble,f_dfLd)
VF4(244,DCfloat,DCfloat,DClonglong,DCdouble,f_ffLd)
VF4(245,DCint,DCint,DCdouble,DCdouble,f_iidd)
VF4(246,DClonglong,DCint,DCdouble,DCdouble,f_Lidd)
VF4(247,DCdouble,DCint,DCdouble,DCdouble,f_didd)
VF4(248,DCfloat,DCint,DCdouble,DCdouble,f_fidd)
VF4(249,DCint,DClonglong,DCdouble,DCdouble,f_iLdd)
VF4(250,DClonglong,DClonglong,DCdouble,DCdouble,f_LLdd)
VF4(251,DCdouble,DClonglong,DCdouble,DCdouble,f_dLdd)
VF4(252,DCfloat,DClonglong,DCdouble,DCdouble,f_fLdd)
VF4(253,DCint,DCdouble,DCdouble,DCdouble,f_iddd)
VF4(254,DClonglong,DCdouble,DCdouble,DCdouble,f_Lddd)
VF4(255,DCdouble,DCdouble,DCdouble,DCdouble,f_dddd)
VF4(256,DCfloat,DCdouble,DCdouble,DCdouble,f_fddd)
VF4(257,DCint,DCfloat,DCdouble,DCdouble,f_ifdd)
VF4(258,DClonglong,DCfloat,DCdouble,DCdouble,f_Lfdd)
VF4(259,DCdouble,DCfloat,DCdouble,DCdouble,f_dfdd)
VF4(260,DCfloat,DCfloat,DCdouble,DCdouble,f_ffdd)
VF4(261,DCint,DCint,DCfloat,DCdouble,f_iifd)
VF4(262,DClonglong,DCint,DCfloat,DCdouble,f_Lifd)
VF4(263,DCdouble,DCint,DCfloat,DCdouble,f_difd)
VF4(264,DCfloat,DCint,DCfloat,DCdouble,f_fifd)
VF4(265,DCint,DClonglong,DCfloat,DCdouble,f_iLfd)
VF4(266,DClonglong,DClonglong,DCfloat,DCdouble,f_LLfd)
VF4(267,DCdouble,DClonglong,DCfloat,DCdouble,f_dLfd)
VF4(268,DCfloat,DClonglong,DCfloat,DCdouble,f_fLfd)
VF4(269,DCint,DCdouble,DCfloat,DCdouble,f_idfd)
VF4(270,DClonglong,DCdouble,DCfloat,DCdouble,f_Ldfd)
VF4(271,DCdouble,DCdouble,DCfloat,DCdouble,f_ddfd)
VF4(272,DCfloat,DCdouble,DCfloat,DCdouble,f_fdfd)
VF4(273,DCint,DCfloat,DCfloat,DCdouble,f_iffd)
VF4(274,DClonglong,DCfloat,DCfloat,DCdouble,f_Lffd)
VF4(275,DCdouble,DCfloat,DCfloat,DCdouble,f_dffd)
VF4(276,DCfloat,DCfloat,DCfloat,DCdouble,f_fffd)
VF4(277,DCint,DCint,DCint,DCfloat,f_iiif)
VF4(278,DClonglong,DCint,DCint,DCfloat,f_Liif)
VF4(279,DCdouble,DCint,DCint,DCfloat,f_diif)
VF4(280,DCfloat,DCint,DCint,DCfloat,f_fiif)
VF4(281,DCint,DClonglong,DCint,DCfloat,f_iLif)
VF4(282,DClonglong,DClonglong,DCint,DCfloat,f_LLif)
VF4(283,DCdouble,DClonglong,DCint,DCfloat,f_dLif)
VF4(284,DCfloat,DClonglong,DCint,DCfloat,f_fLif)
VF4(285,DCint,DCdouble,DCint,DCfloat,f_idif)
VF4(286,DClonglong,DCdouble,DCint,DCfloat,f_Ldif)
VF4(287,DCdouble,DCdouble,DCint,DCfloat,f_ddif)
VF4(288,DCfloat,DCdouble,DCint,DCfloat,f_fdif)
VF4(289,DCint,DCfloat,DCint,DCfloat,f_ifif)
VF4(290,DClonglong,DCfloat,DCint,DCfloat,f_Lfif)
VF4(291,DCdouble,DCfloat,DCint,DCfloat,f_dfif)
VF4(292,DCfloat,DCfloat,DCint,DCfloat,f_ffif)
VF4(293,DCint,DCint,DClonglong,DCfloat,f_iiLf)
VF4(294,DClonglong,DCint,DClonglong,DCfloat,f_LiLf)
VF4(295,DCdouble,DCint,DClonglong,DCfloat,f_diLf)
VF4(296,DCfloat,DCint,DClonglong,DCfloat,f_fiLf)
VF4(297,DCint,DClonglong,DClonglong,DCfloat,f_iLLf)
VF4(298,DClonglong,DClonglong,DClonglong,DCfloat,f_LLLf)
VF4(299,DCdouble,DClonglong,DClonglong,DCfloat,f_dLLf)
VF4(300,DCfloat,DClonglong,DClonglong,DCfloat,f_fLLf)
VF4(301,DCint,DCdouble,DClonglong,DCfloat,f_idLf)
VF4(302,DClonglong,DCdouble,DClonglong,DCfloat,f_LdLf)
VF4(303,DCdouble,DCdouble,DClonglong,DCfloat,f_ddLf)
VF4(304,DCfloat,DCdouble,DClonglong,DCfloat,f_fdLf)
VF4(305,DCint,DCfloat,DClonglong,DCfloat,f_ifLf)
VF4(306,DClonglong,DCfloat,DClonglong,DCfloat,f_LfLf)
VF4(307,DCdouble,DCfloat,DClonglong,DCfloat,f_dfLf)
VF4(308,DCfloat,DCfloat,DClonglong,DCfloat,f_ffLf)
VF4(309,DCint,DCint,DCdouble,DCfloat,f_iidf)
VF4(310,DClonglong,DCint,DCdouble,DCfloat,f_Lidf)
VF4(311,DCdouble,DCint,DCdouble,DCfloat,f_didf)
VF4(312,DCfloat,DCint,DCdouble,DCfloat,f_fidf)
VF4(313,DCint,DClonglong,DCdouble,DCfloat,f_iLdf)
VF4(314,DClonglong,DClonglong,DCdouble,DCfloat,f_LLdf)
VF4(315,DCdouble,DClonglong,DCdouble,DCfloat,f_dLdf)
VF4(316,DCfloat,DClonglong,DCdouble,DCfloat,f_fLdf)
VF4(317,DCint,DCdouble,DCdouble,DCfloat,f_iddf)
VF4(318,DClonglong,DCdouble,DCdouble,DCfloat,f_Lddf)
VF4(319,DCdouble,DCdouble,DCdouble,DCfloat,f_dddf)
VF4(320,DCfloat,DCdouble,DCdouble,DCfloat,f_fddf)
VF4(321,DCint,DCfloat,DCdouble,DCfloat,f_ifdf)
VF4(322,DClonglong,DCfloat,DCdouble,DCfloat,f_Lfdf)
VF4(323,DCdouble,DCfloat,DCdouble,DCfloat,f_dfdf)
VF4(324,DCfloat,DCfloat,DCdouble,DCfloat,f_ffdf)
VF4(325,DCint,DCint,DCfloat,DCfloat,f_iiff)
VF4(326,DClonglong,DCint,DCfloat,DCfloat,f_Liff)
VF4(327,DCdouble,DCint,DCfloat,DCfloat,f_diff)
VF4(328,DCfloat,DCint,DCfloat,DCfloat,f_fiff)
VF4(329,DCint,DClonglong,DCfloat,DCfloat,f_iLff)
VF4(330,DClonglong,DClonglong,DCfloat,DCfloat,f_LLff)
VF4(331,DCdouble,DClonglong,DCfloat,DCfloat,f_dLff)
VF4(332,DCfloat,DClonglong,DCfloat,DCfloat,f_fLff)
VF4(333,DCint,DCdouble,DCfloat,DCfloat,f_idff)
VF4(334,DClonglong,DCdouble,DCfloat,DCfloat,f_Ldff)
VF4(335,DCdouble,DCdouble,DCfloat,DCfloat,f_ddff)
VF4(336,DCfloat,DCdouble,DCfloat,DCfloat,f_fdff)
VF4(337,DCint,DCfloat,DCfloat,DCfloat,f_ifff)
VF4(338,DClonglong,DCfloat,DCfloat,DCfloat,f_Lfff)
VF4(339,DCdouble,DCfloat,DCfloat,DCfloat,f_dfff)
VF4(340,DCfloat,DCfloat,DCfloat,DCfloat,f_ffff)
