/*

 Package: dyncall
 Library: test
 File: test/suite_x86win32fast/case.h
 Description: 
 License:

   Copyright (c) 2011-2018 Daniel Adler <dadler@uni-goettingen.de>,
                           Tassilo Philipp <tphilipp@potion-studios.com>

   Permission to use, copy, modify, and distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
   copyright notice and this permission notice appear in all copies.

   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
   WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
   ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

/* auto generated by mkcase.py (on stdout) */
VF0(0,f_)
VF1(1,DCbool,f_b)
VF1(2,DCint,f_i)
VF1(3,DClonglong,f_l)
VF1(4,DCdouble,f_d)
VF1(5,DCpointer,f_p)
VF1(6,DCfloat,f_f)
VF2(7,DCbool,DCbool,f_bb)
VF2(8,DCint,DCbool,f_ib)
VF2(9,DClonglong,DCbool,f_lb)
VF2(10,DCdouble,DCbool,f_db)
VF2(11,DCpointer,DCbool,f_pb)
VF2(12,DCfloat,DCbool,f_fb)
VF2(13,DCbool,DCint,f_bi)
VF2(14,DCint,DCint,f_ii)
VF2(15,DClonglong,DCint,f_li)
VF2(16,DCdouble,DCint,f_di)
VF2(17,DCpointer,DCint,f_pi)
VF2(18,DCfloat,DCint,f_fi)
VF2(19,DCbool,DClonglong,f_bl)
VF2(20,DCint,DClonglong,f_il)
VF2(21,DClonglong,DClonglong,f_ll)
VF2(22,DCdouble,DClonglong,f_dl)
VF2(23,DCpointer,DClonglong,f_pl)
VF2(24,DCfloat,DClonglong,f_fl)
VF2(25,DCbool,DCdouble,f_bd)
VF2(26,DCint,DCdouble,f_id)
VF2(27,DClonglong,DCdouble,f_ld)
VF2(28,DCdouble,DCdouble,f_dd)
VF2(29,DCpointer,DCdouble,f_pd)
VF2(30,DCfloat,DCdouble,f_fd)
VF2(31,DCbool,DCpointer,f_bp)
VF2(32,DCint,DCpointer,f_ip)
VF2(33,DClonglong,DCpointer,f_lp)
VF2(34,DCdouble,DCpointer,f_dp)
VF2(35,DCpointer,DCpointer,f_pp)
VF2(36,DCfloat,DCpointer,f_fp)
VF2(37,DCbool,DCfloat,f_bf)
VF2(38,DCint,DCfloat,f_if)
VF2(39,DClonglong,DCfloat,f_lf)
VF2(40,DCdouble,DCfloat,f_df)
VF2(41,DCpointer,DCfloat,f_pf)
VF2(42,DCfloat,DCfloat,f_ff)
VF3(43,DCbool,DCbool,DCbool,f_bbb)
VF3(44,DCint,DCbool,DCbool,f_ibb)
VF3(45,DClonglong,DCbool,DCbool,f_lbb)
VF3(46,DCdouble,DCbool,DCbool,f_dbb)
VF3(47,DCpointer,DCbool,DCbool,f_pbb)
VF3(48,DCfloat,DCbool,DCbool,f_fbb)
VF3(49,DCbool,DCint,DCbool,f_bib)
VF3(50,DCint,DCint,DCbool,f_iib)
VF3(51,DClonglong,DCint,DCbool,f_lib)
VF3(52,DCdouble,DCint,DCbool,f_dib)
VF3(53,DCpointer,DCint,DCbool,f_pib)
VF3(54,DCfloat,DCint,DCbool,f_fib)
VF3(55,DCbool,DClonglong,DCbool,f_blb)
VF3(56,DCint,DClonglong,DCbool,f_ilb)
VF3(57,DClonglong,DClonglong,DCbool,f_llb)
VF3(58,DCdouble,DClonglong,DCbool,f_dlb)
VF3(59,DCpointer,DClonglong,DCbool,f_plb)
VF3(60,DCfloat,DClonglong,DCbool,f_flb)
VF3(61,DCbool,DCdouble,DCbool,f_bdb)
VF3(62,DCint,DCdouble,DCbool,f_idb)
VF3(63,DClonglong,DCdouble,DCbool,f_ldb)
VF3(64,DCdouble,DCdouble,DCbool,f_ddb)
VF3(65,DCpointer,DCdouble,DCbool,f_pdb)
VF3(66,DCfloat,DCdouble,DCbool,f_fdb)
VF3(67,DCbool,DCpointer,DCbool,f_bpb)
VF3(68,DCint,DCpointer,DCbool,f_ipb)
VF3(69,DClonglong,DCpointer,DCbool,f_lpb)
VF3(70,DCdouble,DCpointer,DCbool,f_dpb)
VF3(71,DCpointer,DCpointer,DCbool,f_ppb)
VF3(72,DCfloat,DCpointer,DCbool,f_fpb)
VF3(73,DCbool,DCfloat,DCbool,f_bfb)
VF3(74,DCint,DCfloat,DCbool,f_ifb)
VF3(75,DClonglong,DCfloat,DCbool,f_lfb)
VF3(76,DCdouble,DCfloat,DCbool,f_dfb)
VF3(77,DCpointer,DCfloat,DCbool,f_pfb)
VF3(78,DCfloat,DCfloat,DCbool,f_ffb)
VF3(79,DCbool,DCbool,DCint,f_bbi)
VF3(80,DCint,DCbool,DCint,f_ibi)
VF3(81,DClonglong,DCbool,DCint,f_lbi)
VF3(82,DCdouble,DCbool,DCint,f_dbi)
VF3(83,DCpointer,DCbool,DCint,f_pbi)
VF3(84,DCfloat,DCbool,DCint,f_fbi)
VF3(85,DCbool,DCint,DCint,f_bii)
VF3(86,DCint,DCint,DCint,f_iii)
VF3(87,DClonglong,DCint,DCint,f_lii)
VF3(88,DCdouble,DCint,DCint,f_dii)
VF3(89,DCpointer,DCint,DCint,f_pii)
VF3(90,DCfloat,DCint,DCint,f_fii)
VF3(91,DCbool,DClonglong,DCint,f_bli)
VF3(92,DCint,DClonglong,DCint,f_ili)
VF3(93,DClonglong,DClonglong,DCint,f_lli)
VF3(94,DCdouble,DClonglong,DCint,f_dli)
VF3(95,DCpointer,DClonglong,DCint,f_pli)
VF3(96,DCfloat,DClonglong,DCint,f_fli)
VF3(97,DCbool,DCdouble,DCint,f_bdi)
VF3(98,DCint,DCdouble,DCint,f_idi)
VF3(99,DClonglong,DCdouble,DCint,f_ldi)
VF3(100,DCdouble,DCdouble,DCint,f_ddi)
VF3(101,DCpointer,DCdouble,DCint,f_pdi)
VF3(102,DCfloat,DCdouble,DCint,f_fdi)
VF3(103,DCbool,DCpointer,DCint,f_bpi)
VF3(104,DCint,DCpointer,DCint,f_ipi)
VF3(105,DClonglong,DCpointer,DCint,f_lpi)
VF3(106,DCdouble,DCpointer,DCint,f_dpi)
VF3(107,DCpointer,DCpointer,DCint,f_ppi)
VF3(108,DCfloat,DCpointer,DCint,f_fpi)
VF3(109,DCbool,DCfloat,DCint,f_bfi)
VF3(110,DCint,DCfloat,DCint,f_ifi)
VF3(111,DClonglong,DCfloat,DCint,f_lfi)
VF3(112,DCdouble,DCfloat,DCint,f_dfi)
VF3(113,DCpointer,DCfloat,DCint,f_pfi)
VF3(114,DCfloat,DCfloat,DCint,f_ffi)
VF3(115,DCbool,DCbool,DClonglong,f_bbl)
VF3(116,DCint,DCbool,DClonglong,f_ibl)
VF3(117,DClonglong,DCbool,DClonglong,f_lbl)
VF3(118,DCdouble,DCbool,DClonglong,f_dbl)
VF3(119,DCpointer,DCbool,DClonglong,f_pbl)
VF3(120,DCfloat,DCbool,DClonglong,f_fbl)
VF3(121,DCbool,DCint,DClonglong,f_bil)
VF3(122,DCint,DCint,DClonglong,f_iil)
VF3(123,DClonglong,DCint,DClonglong,f_lil)
VF3(124,DCdouble,DCint,DClonglong,f_dil)
VF3(125,DCpointer,DCint,DClonglong,f_pil)
VF3(126,DCfloat,DCint,DClonglong,f_fil)
VF3(127,DCbool,DClonglong,DClonglong,f_bll)
VF3(128,DCint,DClonglong,DClonglong,f_ill)
VF3(129,DClonglong,DClonglong,DClonglong,f_lll)
VF3(130,DCdouble,DClonglong,DClonglong,f_dll)
VF3(131,DCpointer,DClonglong,DClonglong,f_pll)
VF3(132,DCfloat,DClonglong,DClonglong,f_fll)
VF3(133,DCbool,DCdouble,DClonglong,f_bdl)
VF3(134,DCint,DCdouble,DClonglong,f_idl)
VF3(135,DClonglong,DCdouble,DClonglong,f_ldl)
VF3(136,DCdouble,DCdouble,DClonglong,f_ddl)
VF3(137,DCpointer,DCdouble,DClonglong,f_pdl)
VF3(138,DCfloat,DCdouble,DClonglong,f_fdl)
VF3(139,DCbool,DCpointer,DClonglong,f_bpl)
VF3(140,DCint,DCpointer,DClonglong,f_ipl)
VF3(141,DClonglong,DCpointer,DClonglong,f_lpl)
VF3(142,DCdouble,DCpointer,DClonglong,f_dpl)
VF3(143,DCpointer,DCpointer,DClonglong,f_ppl)
VF3(144,DCfloat,DCpointer,DClonglong,f_fpl)
VF3(145,DCbool,DCfloat,DClonglong,f_bfl)
VF3(146,DCint,DCfloat,DClonglong,f_ifl)
VF3(147,DClonglong,DCfloat,DClonglong,f_lfl)
VF3(148,DCdouble,DCfloat,DClonglong,f_dfl)
VF3(149,DCpointer,DCfloat,DClonglong,f_pfl)
VF3(150,DCfloat,DCfloat,DClonglong,f_ffl)
VF3(151,DCbool,DCbool,DCdouble,f_bbd)
VF3(152,DCint,DCbool,DCdouble,f_ibd)
VF3(153,DClonglong,DCbool,DCdouble,f_lbd)
VF3(154,DCdouble,DCbool,DCdouble,f_dbd)
VF3(155,DCpointer,DCbool,DCdouble,f_pbd)
VF3(156,DCfloat,DCbool,DCdouble,f_fbd)
VF3(157,DCbool,DCint,DCdouble,f_bid)
VF3(158,DCint,DCint,DCdouble,f_iid)
VF3(159,DClonglong,DCint,DCdouble,f_lid)
VF3(160,DCdouble,DCint,DCdouble,f_did)
VF3(161,DCpointer,DCint,DCdouble,f_pid)
VF3(162,DCfloat,DCint,DCdouble,f_fid)
VF3(163,DCbool,DClonglong,DCdouble,f_bld)
VF3(164,DCint,DClonglong,DCdouble,f_ild)
VF3(165,DClonglong,DClonglong,DCdouble,f_lld)
VF3(166,DCdouble,DClonglong,DCdouble,f_dld)
VF3(167,DCpointer,DClonglong,DCdouble,f_pld)
VF3(168,DCfloat,DClonglong,DCdouble,f_fld)
VF3(169,DCbool,DCdouble,DCdouble,f_bdd)
VF3(170,DCint,DCdouble,DCdouble,f_idd)
VF3(171,DClonglong,DCdouble,DCdouble,f_ldd)
VF3(172,DCdouble,DCdouble,DCdouble,f_ddd)
VF3(173,DCpointer,DCdouble,DCdouble,f_pdd)
VF3(174,DCfloat,DCdouble,DCdouble,f_fdd)
VF3(175,DCbool,DCpointer,DCdouble,f_bpd)
VF3(176,DCint,DCpointer,DCdouble,f_ipd)
VF3(177,DClonglong,DCpointer,DCdouble,f_lpd)
VF3(178,DCdouble,DCpointer,DCdouble,f_dpd)
VF3(179,DCpointer,DCpointer,DCdouble,f_ppd)
VF3(180,DCfloat,DCpointer,DCdouble,f_fpd)
VF3(181,DCbool,DCfloat,DCdouble,f_bfd)
VF3(182,DCint,DCfloat,DCdouble,f_ifd)
VF3(183,DClonglong,DCfloat,DCdouble,f_lfd)
VF3(184,DCdouble,DCfloat,DCdouble,f_dfd)
VF3(185,DCpointer,DCfloat,DCdouble,f_pfd)
VF3(186,DCfloat,DCfloat,DCdouble,f_ffd)
VF3(187,DCbool,DCbool,DCpointer,f_bbp)
VF3(188,DCint,DCbool,DCpointer,f_ibp)
VF3(189,DClonglong,DCbool,DCpointer,f_lbp)
VF3(190,DCdouble,DCbool,DCpointer,f_dbp)
VF3(191,DCpointer,DCbool,DCpointer,f_pbp)
VF3(192,DCfloat,DCbool,DCpointer,f_fbp)
VF3(193,DCbool,DCint,DCpointer,f_bip)
VF3(194,DCint,DCint,DCpointer,f_iip)
VF3(195,DClonglong,DCint,DCpointer,f_lip)
VF3(196,DCdouble,DCint,DCpointer,f_dip)
VF3(197,DCpointer,DCint,DCpointer,f_pip)
VF3(198,DCfloat,DCint,DCpointer,f_fip)
VF3(199,DCbool,DClonglong,DCpointer,f_blp)
VF3(200,DCint,DClonglong,DCpointer,f_ilp)
VF3(201,DClonglong,DClonglong,DCpointer,f_llp)
VF3(202,DCdouble,DClonglong,DCpointer,f_dlp)
VF3(203,DCpointer,DClonglong,DCpointer,f_plp)
VF3(204,DCfloat,DClonglong,DCpointer,f_flp)
VF3(205,DCbool,DCdouble,DCpointer,f_bdp)
VF3(206,DCint,DCdouble,DCpointer,f_idp)
VF3(207,DClonglong,DCdouble,DCpointer,f_ldp)
VF3(208,DCdouble,DCdouble,DCpointer,f_ddp)
VF3(209,DCpointer,DCdouble,DCpointer,f_pdp)
VF3(210,DCfloat,DCdouble,DCpointer,f_fdp)
VF3(211,DCbool,DCpointer,DCpointer,f_bpp)
VF3(212,DCint,DCpointer,DCpointer,f_ipp)
VF3(213,DClonglong,DCpointer,DCpointer,f_lpp)
VF3(214,DCdouble,DCpointer,DCpointer,f_dpp)
VF3(215,DCpointer,DCpointer,DCpointer,f_ppp)
VF3(216,DCfloat,DCpointer,DCpointer,f_fpp)
VF3(217,DCbool,DCfloat,DCpointer,f_bfp)
VF3(218,DCint,DCfloat,DCpointer,f_ifp)
VF3(219,DClonglong,DCfloat,DCpointer,f_lfp)
VF3(220,DCdouble,DCfloat,DCpointer,f_dfp)
VF3(221,DCpointer,DCfloat,DCpointer,f_pfp)
VF3(222,DCfloat,DCfloat,DCpointer,f_ffp)
VF3(223,DCbool,DCbool,DCfloat,f_bbf)
VF3(224,DCint,DCbool,DCfloat,f_ibf)
VF3(225,DClonglong,DCbool,DCfloat,f_lbf)
VF3(226,DCdouble,DCbool,DCfloat,f_dbf)
VF3(227,DCpointer,DCbool,DCfloat,f_pbf)
VF3(228,DCfloat,DCbool,DCfloat,f_fbf)
VF3(229,DCbool,DCint,DCfloat,f_bif)
VF3(230,DCint,DCint,DCfloat,f_iif)
VF3(231,DClonglong,DCint,DCfloat,f_lif)
VF3(232,DCdouble,DCint,DCfloat,f_dif)
VF3(233,DCpointer,DCint,DCfloat,f_pif)
VF3(234,DCfloat,DCint,DCfloat,f_fif)
VF3(235,DCbool,DClonglong,DCfloat,f_blf)
VF3(236,DCint,DClonglong,DCfloat,f_ilf)
VF3(237,DClonglong,DClonglong,DCfloat,f_llf)
VF3(238,DCdouble,DClonglong,DCfloat,f_dlf)
VF3(239,DCpointer,DClonglong,DCfloat,f_plf)
VF3(240,DCfloat,DClonglong,DCfloat,f_flf)
VF3(241,DCbool,DCdouble,DCfloat,f_bdf)
VF3(242,DCint,DCdouble,DCfloat,f_idf)
VF3(243,DClonglong,DCdouble,DCfloat,f_ldf)
VF3(244,DCdouble,DCdouble,DCfloat,f_ddf)
VF3(245,DCpointer,DCdouble,DCfloat,f_pdf)
VF3(246,DCfloat,DCdouble,DCfloat,f_fdf)
VF3(247,DCbool,DCpointer,DCfloat,f_bpf)
VF3(248,DCint,DCpointer,DCfloat,f_ipf)
VF3(249,DClonglong,DCpointer,DCfloat,f_lpf)
VF3(250,DCdouble,DCpointer,DCfloat,f_dpf)
VF3(251,DCpointer,DCpointer,DCfloat,f_ppf)
VF3(252,DCfloat,DCpointer,DCfloat,f_fpf)
VF3(253,DCbool,DCfloat,DCfloat,f_bff)
VF3(254,DCint,DCfloat,DCfloat,f_iff)
VF3(255,DClonglong,DCfloat,DCfloat,f_lff)
VF3(256,DCdouble,DCfloat,DCfloat,f_dff)
VF3(257,DCpointer,DCfloat,DCfloat,f_pff)
VF3(258,DCfloat,DCfloat,DCfloat,f_fff)
