/*

 Package: dyncall
 Library: test
 File: test/suite/case.h
 Description: 
 License:

   Copyright (c) 2011-2018 Daniel Adler <dadler@uni-goettingen.de>,
                           Tassilo Philipp <tphilipp@potion-studios.com>

   Permission to use, copy, modify, and distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
   copyright notice and this permission notice appear in all copies.

   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
   WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
   ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

/* auto generated by mkcase.py (on stdout) */
VF0(0,f_)
VF1(1,DCdouble,f_d)
VF1(2,DClonglong,f_l)
VF2(3,DCdouble,DCdouble,f_dd)
VF2(4,DClonglong,DCdouble,f_ld)
VF2(5,DCdouble,DClonglong,f_dl)
VF2(6,DClonglong,DClonglong,f_ll)
VF3(7,DCdouble,DCdouble,DCdouble,f_ddd)
VF3(8,DClonglong,DCdouble,DCdouble,f_ldd)
VF3(9,DCdouble,DClonglong,DCdouble,f_dld)
VF3(10,DClonglong,DClonglong,DCdouble,f_lld)
VF3(11,DCdouble,DCdouble,DClonglong,f_ddl)
VF3(12,DClonglong,DCdouble,DClonglong,f_ldl)
VF3(13,DCdouble,DClonglong,DClonglong,f_dll)
VF3(14,DClonglong,DClonglong,DClonglong,f_lll)
VF4(15,DCdouble,DCdouble,DCdouble,DCdouble,f_dddd)
VF4(16,DClonglong,DCdouble,DCdouble,DCdouble,f_lddd)
VF4(17,DCdouble,DClonglong,DCdouble,DCdouble,f_dldd)
VF4(18,DClonglong,DClonglong,DCdouble,DCdouble,f_lldd)
VF4(19,DCdouble,DCdouble,DClonglong,DCdouble,f_ddld)
VF4(20,DClonglong,DCdouble,DClonglong,DCdouble,f_ldld)
VF4(21,DCdouble,DClonglong,DClonglong,DCdouble,f_dlld)
VF4(22,DClonglong,DClonglong,DClonglong,DCdouble,f_llld)
VF4(23,DCdouble,DCdouble,DCdouble,DClonglong,f_dddl)
VF4(24,DClonglong,DCdouble,DCdouble,DClonglong,f_lddl)
VF4(25,DCdouble,DClonglong,DCdouble,DClonglong,f_dldl)
VF4(26,DClonglong,DClonglong,DCdouble,DClonglong,f_lldl)
VF4(27,DCdouble,DCdouble,DClonglong,DClonglong,f_ddll)
VF4(28,DClonglong,DCdouble,DClonglong,DClonglong,f_ldll)
VF4(29,DCdouble,DClonglong,DClonglong,DClonglong,f_dlll)
VF4(30,DClonglong,DClonglong,DClonglong,DClonglong,f_llll)
VF5(31,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_ddddd)
VF5(32,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_ldddd)
VF5(33,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_dlddd)
VF5(34,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_llddd)
VF5(35,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_ddldd)
VF5(36,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_ldldd)
VF5(37,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_dlldd)
VF5(38,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_llldd)
VF5(39,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_dddld)
VF5(40,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_lddld)
VF5(41,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_dldld)
VF5(42,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_lldld)
VF5(43,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_ddlld)
VF5(44,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_ldlld)
VF5(45,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_dllld)
VF5(46,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_lllld)
VF5(47,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_ddddl)
VF5(48,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_ldddl)
VF5(49,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_dlddl)
VF5(50,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_llddl)
VF5(51,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_ddldl)
VF5(52,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_ldldl)
VF5(53,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_dlldl)
VF5(54,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_llldl)
VF5(55,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_dddll)
VF5(56,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_lddll)
VF5(57,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_dldll)
VF5(58,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_lldll)
VF5(59,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_ddlll)
VF5(60,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_ldlll)
VF5(61,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_dllll)
VF5(62,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_lllll)
VF6(63,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_dddddd)
VF6(64,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_lddddd)
VF6(65,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_dldddd)
VF6(66,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_lldddd)
VF6(67,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_ddlddd)
VF6(68,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_ldlddd)
VF6(69,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_dllddd)
VF6(70,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_lllddd)
VF6(71,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_dddldd)
VF6(72,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_lddldd)
VF6(73,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_dldldd)
VF6(74,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_lldldd)
VF6(75,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_ddlldd)
VF6(76,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_ldlldd)
VF6(77,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_dllldd)
VF6(78,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_lllldd)
VF6(79,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_ddddld)
VF6(80,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_ldddld)
VF6(81,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_dlddld)
VF6(82,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_llddld)
VF6(83,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_ddldld)
VF6(84,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_ldldld)
VF6(85,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_dlldld)
VF6(86,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_llldld)
VF6(87,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_dddlld)
VF6(88,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_lddlld)
VF6(89,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_dldlld)
VF6(90,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_lldlld)
VF6(91,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_ddllld)
VF6(92,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_ldllld)
VF6(93,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_dlllld)
VF6(94,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_llllld)
VF6(95,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_dddddl)
VF6(96,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_lddddl)
VF6(97,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_dldddl)
VF6(98,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_lldddl)
VF6(99,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_ddlddl)
VF6(100,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_ldlddl)
VF6(101,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_dllddl)
VF6(102,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_lllddl)
VF6(103,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_dddldl)
VF6(104,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_lddldl)
VF6(105,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_dldldl)
VF6(106,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_lldldl)
VF6(107,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_ddlldl)
VF6(108,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_ldlldl)
VF6(109,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_dllldl)
VF6(110,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_lllldl)
VF6(111,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_ddddll)
VF6(112,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_ldddll)
VF6(113,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_dlddll)
VF6(114,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_llddll)
VF6(115,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_ddldll)
VF6(116,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_ldldll)
VF6(117,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_dlldll)
VF6(118,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_llldll)
VF6(119,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_dddlll)
VF6(120,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_lddlll)
VF6(121,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_dldlll)
VF6(122,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_lldlll)
VF6(123,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_ddllll)
VF6(124,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_ldllll)
VF6(125,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_dlllll)
VF6(126,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_llllll)
VF7(127,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_ddddddd)
VF7(128,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_ldddddd)
VF7(129,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_dlddddd)
VF7(130,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_llddddd)
VF7(131,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_ddldddd)
VF7(132,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_ldldddd)
VF7(133,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_dlldddd)
VF7(134,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_llldddd)
VF7(135,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_dddlddd)
VF7(136,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_lddlddd)
VF7(137,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_dldlddd)
VF7(138,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_lldlddd)
VF7(139,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_ddllddd)
VF7(140,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_ldllddd)
VF7(141,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_dlllddd)
VF7(142,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_llllddd)
VF7(143,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_ddddldd)
VF7(144,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_ldddldd)
VF7(145,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_dlddldd)
VF7(146,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_llddldd)
VF7(147,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_ddldldd)
VF7(148,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_ldldldd)
VF7(149,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_dlldldd)
VF7(150,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_llldldd)
VF7(151,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_dddlldd)
VF7(152,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_lddlldd)
VF7(153,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_dldlldd)
VF7(154,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_lldlldd)
VF7(155,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_ddllldd)
VF7(156,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_ldllldd)
VF7(157,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_dlllldd)
VF7(158,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_llllldd)
VF7(159,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_dddddld)
VF7(160,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_lddddld)
VF7(161,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_dldddld)
VF7(162,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_lldddld)
VF7(163,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_ddlddld)
VF7(164,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_ldlddld)
VF7(165,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_dllddld)
VF7(166,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_lllddld)
VF7(167,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_dddldld)
VF7(168,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_lddldld)
VF7(169,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_dldldld)
VF7(170,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_lldldld)
VF7(171,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_ddlldld)
VF7(172,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_ldlldld)
VF7(173,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_dllldld)
VF7(174,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_lllldld)
VF7(175,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_ddddlld)
VF7(176,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_ldddlld)
VF7(177,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_dlddlld)
VF7(178,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_llddlld)
VF7(179,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_ddldlld)
VF7(180,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_ldldlld)
VF7(181,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_dlldlld)
VF7(182,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_llldlld)
VF7(183,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_dddllld)
VF7(184,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_lddllld)
VF7(185,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_dldllld)
VF7(186,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_lldllld)
VF7(187,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_ddlllld)
VF7(188,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_ldlllld)
VF7(189,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_dllllld)
VF7(190,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_lllllld)
VF7(191,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_ddddddl)
VF7(192,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_ldddddl)
VF7(193,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_dlddddl)
VF7(194,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_llddddl)
VF7(195,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_ddldddl)
VF7(196,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_ldldddl)
VF7(197,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_dlldddl)
VF7(198,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_llldddl)
VF7(199,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_dddlddl)
VF7(200,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_lddlddl)
VF7(201,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_dldlddl)
VF7(202,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_lldlddl)
VF7(203,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_ddllddl)
VF7(204,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_ldllddl)
VF7(205,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_dlllddl)
VF7(206,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_llllddl)
VF7(207,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_ddddldl)
VF7(208,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_ldddldl)
VF7(209,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_dlddldl)
VF7(210,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_llddldl)
VF7(211,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_ddldldl)
VF7(212,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_ldldldl)
VF7(213,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_dlldldl)
VF7(214,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_llldldl)
VF7(215,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_dddlldl)
VF7(216,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_lddlldl)
VF7(217,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_dldlldl)
VF7(218,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_lldlldl)
VF7(219,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_ddllldl)
VF7(220,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_ldllldl)
VF7(221,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_dlllldl)
VF7(222,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_llllldl)
VF7(223,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_dddddll)
VF7(224,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_lddddll)
VF7(225,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_dldddll)
VF7(226,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_lldddll)
VF7(227,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_ddlddll)
VF7(228,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_ldlddll)
VF7(229,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_dllddll)
VF7(230,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_lllddll)
VF7(231,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_dddldll)
VF7(232,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_lddldll)
VF7(233,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_dldldll)
VF7(234,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_lldldll)
VF7(235,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_ddlldll)
VF7(236,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_ldlldll)
VF7(237,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_dllldll)
VF7(238,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_lllldll)
VF7(239,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_ddddlll)
VF7(240,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_ldddlll)
VF7(241,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_dlddlll)
VF7(242,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_llddlll)
VF7(243,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_ddldlll)
VF7(244,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_ldldlll)
VF7(245,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_dlldlll)
VF7(246,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_llldlll)
VF7(247,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_dddllll)
VF7(248,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_lddllll)
VF7(249,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_dldllll)
VF7(250,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_lldllll)
VF7(251,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_ddlllll)
VF7(252,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_ldlllll)
VF7(253,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_dllllll)
VF7(254,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_lllllll)
VF8(255,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_dddddddd)
VF8(256,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_lddddddd)
VF8(257,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_dldddddd)
VF8(258,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_lldddddd)
VF8(259,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_ddlddddd)
VF8(260,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_ldlddddd)
VF8(261,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_dllddddd)
VF8(262,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,f_lllddddd)
VF8(263,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_dddldddd)
VF8(264,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_lddldddd)
VF8(265,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_dldldddd)
VF8(266,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_lldldddd)
VF8(267,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_ddlldddd)
VF8(268,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_ldlldddd)
VF8(269,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_dllldddd)
VF8(270,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,f_lllldddd)
VF8(271,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_ddddlddd)
VF8(272,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_ldddlddd)
VF8(273,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_dlddlddd)
VF8(274,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_llddlddd)
VF8(275,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_ddldlddd)
VF8(276,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_ldldlddd)
VF8(277,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_dlldlddd)
VF8(278,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,f_llldlddd)
VF8(279,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_dddllddd)
VF8(280,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_lddllddd)
VF8(281,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_dldllddd)
VF8(282,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_lldllddd)
VF8(283,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_ddlllddd)
VF8(284,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_ldlllddd)
VF8(285,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_dllllddd)
VF8(286,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,f_lllllddd)
VF8(287,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_dddddldd)
VF8(288,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_lddddldd)
VF8(289,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_dldddldd)
VF8(290,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_lldddldd)
VF8(291,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_ddlddldd)
VF8(292,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_ldlddldd)
VF8(293,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_dllddldd)
VF8(294,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,f_lllddldd)
VF8(295,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_dddldldd)
VF8(296,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_lddldldd)
VF8(297,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_dldldldd)
VF8(298,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_lldldldd)
VF8(299,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_ddlldldd)
VF8(300,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_ldlldldd)
VF8(301,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_dllldldd)
VF8(302,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,f_lllldldd)
VF8(303,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_ddddlldd)
VF8(304,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_ldddlldd)
VF8(305,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_dlddlldd)
VF8(306,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_llddlldd)
VF8(307,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_ddldlldd)
VF8(308,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_ldldlldd)
VF8(309,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_dlldlldd)
VF8(310,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,f_llldlldd)
VF8(311,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_dddllldd)
VF8(312,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_lddllldd)
VF8(313,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_dldllldd)
VF8(314,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_lldllldd)
VF8(315,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_ddlllldd)
VF8(316,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_ldlllldd)
VF8(317,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_dllllldd)
VF8(318,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,f_lllllldd)
VF8(319,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_ddddddld)
VF8(320,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_ldddddld)
VF8(321,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_dlddddld)
VF8(322,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_llddddld)
VF8(323,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_ddldddld)
VF8(324,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_ldldddld)
VF8(325,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_dlldddld)
VF8(326,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,f_llldddld)
VF8(327,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_dddlddld)
VF8(328,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_lddlddld)
VF8(329,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_dldlddld)
VF8(330,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_lldlddld)
VF8(331,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_ddllddld)
VF8(332,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_ldllddld)
VF8(333,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_dlllddld)
VF8(334,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,f_llllddld)
VF8(335,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_ddddldld)
VF8(336,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_ldddldld)
VF8(337,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_dlddldld)
VF8(338,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_llddldld)
VF8(339,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_ddldldld)
VF8(340,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_ldldldld)
VF8(341,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_dlldldld)
VF8(342,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,f_llldldld)
VF8(343,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_dddlldld)
VF8(344,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_lddlldld)
VF8(345,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_dldlldld)
VF8(346,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_lldlldld)
VF8(347,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_ddllldld)
VF8(348,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_ldllldld)
VF8(349,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_dlllldld)
VF8(350,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,f_llllldld)
VF8(351,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_dddddlld)
VF8(352,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_lddddlld)
VF8(353,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_dldddlld)
VF8(354,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_lldddlld)
VF8(355,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_ddlddlld)
VF8(356,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_ldlddlld)
VF8(357,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_dllddlld)
VF8(358,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,f_lllddlld)
VF8(359,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_dddldlld)
VF8(360,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_lddldlld)
VF8(361,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_dldldlld)
VF8(362,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_lldldlld)
VF8(363,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_ddlldlld)
VF8(364,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_ldlldlld)
VF8(365,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_dllldlld)
VF8(366,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,f_lllldlld)
VF8(367,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_ddddllld)
VF8(368,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_ldddllld)
VF8(369,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_dlddllld)
VF8(370,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_llddllld)
VF8(371,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_ddldllld)
VF8(372,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_ldldllld)
VF8(373,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_dlldllld)
VF8(374,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,f_llldllld)
VF8(375,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_dddlllld)
VF8(376,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_lddlllld)
VF8(377,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_dldlllld)
VF8(378,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_lldlllld)
VF8(379,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_ddllllld)
VF8(380,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_ldllllld)
VF8(381,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_dlllllld)
VF8(382,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,f_llllllld)
VF8(383,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_dddddddl)
VF8(384,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_lddddddl)
VF8(385,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_dldddddl)
VF8(386,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_lldddddl)
VF8(387,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_ddlddddl)
VF8(388,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_ldlddddl)
VF8(389,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_dllddddl)
VF8(390,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,f_lllddddl)
VF8(391,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_dddldddl)
VF8(392,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_lddldddl)
VF8(393,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_dldldddl)
VF8(394,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_lldldddl)
VF8(395,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_ddlldddl)
VF8(396,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_ldlldddl)
VF8(397,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_dllldddl)
VF8(398,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,f_lllldddl)
VF8(399,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_ddddlddl)
VF8(400,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_ldddlddl)
VF8(401,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_dlddlddl)
VF8(402,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_llddlddl)
VF8(403,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_ddldlddl)
VF8(404,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_ldldlddl)
VF8(405,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_dlldlddl)
VF8(406,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,f_llldlddl)
VF8(407,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_dddllddl)
VF8(408,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_lddllddl)
VF8(409,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_dldllddl)
VF8(410,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_lldllddl)
VF8(411,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_ddlllddl)
VF8(412,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_ldlllddl)
VF8(413,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_dllllddl)
VF8(414,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,f_lllllddl)
VF8(415,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_dddddldl)
VF8(416,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_lddddldl)
VF8(417,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_dldddldl)
VF8(418,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_lldddldl)
VF8(419,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_ddlddldl)
VF8(420,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_ldlddldl)
VF8(421,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_dllddldl)
VF8(422,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,f_lllddldl)
VF8(423,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_dddldldl)
VF8(424,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_lddldldl)
VF8(425,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_dldldldl)
VF8(426,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_lldldldl)
VF8(427,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_ddlldldl)
VF8(428,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_ldlldldl)
VF8(429,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_dllldldl)
VF8(430,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,f_lllldldl)
VF8(431,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_ddddlldl)
VF8(432,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_ldddlldl)
VF8(433,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_dlddlldl)
VF8(434,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_llddlldl)
VF8(435,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_ddldlldl)
VF8(436,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_ldldlldl)
VF8(437,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_dlldlldl)
VF8(438,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,f_llldlldl)
VF8(439,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_dddllldl)
VF8(440,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_lddllldl)
VF8(441,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_dldllldl)
VF8(442,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_lldllldl)
VF8(443,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_ddlllldl)
VF8(444,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_ldlllldl)
VF8(445,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_dllllldl)
VF8(446,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,f_lllllldl)
VF8(447,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_ddddddll)
VF8(448,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_ldddddll)
VF8(449,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_dlddddll)
VF8(450,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_llddddll)
VF8(451,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_ddldddll)
VF8(452,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_ldldddll)
VF8(453,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_dlldddll)
VF8(454,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,f_llldddll)
VF8(455,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_dddlddll)
VF8(456,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_lddlddll)
VF8(457,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_dldlddll)
VF8(458,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_lldlddll)
VF8(459,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_ddllddll)
VF8(460,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_ldllddll)
VF8(461,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_dlllddll)
VF8(462,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,f_llllddll)
VF8(463,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_ddddldll)
VF8(464,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_ldddldll)
VF8(465,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_dlddldll)
VF8(466,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_llddldll)
VF8(467,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_ddldldll)
VF8(468,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_ldldldll)
VF8(469,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_dlldldll)
VF8(470,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,f_llldldll)
VF8(471,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_dddlldll)
VF8(472,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_lddlldll)
VF8(473,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_dldlldll)
VF8(474,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_lldlldll)
VF8(475,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_ddllldll)
VF8(476,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_ldllldll)
VF8(477,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_dlllldll)
VF8(478,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,f_llllldll)
VF8(479,DCdouble,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_dddddlll)
VF8(480,DClonglong,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_lddddlll)
VF8(481,DCdouble,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_dldddlll)
VF8(482,DClonglong,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_lldddlll)
VF8(483,DCdouble,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_ddlddlll)
VF8(484,DClonglong,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_ldlddlll)
VF8(485,DCdouble,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_dllddlll)
VF8(486,DClonglong,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,f_lllddlll)
VF8(487,DCdouble,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_dddldlll)
VF8(488,DClonglong,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_lddldlll)
VF8(489,DCdouble,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_dldldlll)
VF8(490,DClonglong,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_lldldlll)
VF8(491,DCdouble,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_ddlldlll)
VF8(492,DClonglong,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_ldlldlll)
VF8(493,DCdouble,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_dllldlll)
VF8(494,DClonglong,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,f_lllldlll)
VF8(495,DCdouble,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_ddddllll)
VF8(496,DClonglong,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_ldddllll)
VF8(497,DCdouble,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_dlddllll)
VF8(498,DClonglong,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_llddllll)
VF8(499,DCdouble,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_ddldllll)
VF8(500,DClonglong,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_ldldllll)
VF8(501,DCdouble,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_dlldllll)
VF8(502,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,f_llldllll)
VF8(503,DCdouble,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_dddlllll)
VF8(504,DClonglong,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_lddlllll)
VF8(505,DCdouble,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_dldlllll)
VF8(506,DClonglong,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_lldlllll)
VF8(507,DCdouble,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_ddllllll)
VF8(508,DClonglong,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_ldllllll)
VF8(509,DCdouble,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_dlllllll)
VF8(510,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,DClonglong,f_llllllll)
