/*

 Package: dyncall
 Library: test
 File: test/suite2_x86win32fast/case.h
 Description: 
 License:

   Copyright (c) 2011-2018 Daniel Adler <dadler@uni-goettingen.de>,
                           Tassilo Philipp <tphilipp@potion-studios.com>

   Permission to use, copy, modify, and distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
   copyright notice and this permission notice appear in all copies.

   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
   WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
   ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

/* auto generated by mkcase.py (on stdout) */
VF18(0,DCpointer,DCshort,DCint,DCpointer,DCshort,DCint,DClonglong,DCint,DCchar,DCbool,DCint,DCpointer,DCdouble,DCfloat,DCdouble,DCint,DCdouble,DCint,s_psipsilicBipdfdidi)
VF13(1,DCshort,DCfloat,DCshort,DCint,DCint,DCdouble,DCint,DClonglong,DCbool,DCpointer,DCdouble,DCdouble,DClonglong,s_sfsiidilBpddl)
VF7(2,DCchar,DCchar,DCchar,DCbool,DCbool,DCchar,DCchar,s_cccBBcc)
VF18(3,DCbool,DCpointer,DClonglong,DCdouble,DCfloat,DClonglong,DCpointer,DCshort,DCchar,DCint,DCshort,DCint,DCdouble,DCbool,DCpointer,DCfloat,DCfloat,DCdouble,s_BpldflpscisidBpffd)
VF7(4,DCint,DCchar,DCdouble,DCdouble,DClonglong,DCfloat,DCshort,s_icddlfs)
VF8(5,DCbool,DCdouble,DCfloat,DCpointer,DCfloat,DCbool,DCshort,DCshort,s_BdfpfBss)
VF1(6,DCint,s_i)
VF3(7,DCbool,DCfloat,DCchar,s_Bfc)
VF18(8,DCchar,DCfloat,DCpointer,DClonglong,DClonglong,DCdouble,DCchar,DCdouble,DCpointer,DClonglong,DClonglong,DClonglong,DClonglong,DCpointer,DCchar,DCpointer,DCfloat,DClonglong,s_cfplldcdpllllpcpfl)
VF5(9,DCfloat,DCpointer,DCchar,DCbool,DCdouble,s_fpcBd)
VF15(10,DClonglong,DCshort,DClonglong,DCfloat,DCpointer,DCchar,DClonglong,DCpointer,DCint,DCfloat,DCint,DCfloat,DCfloat,DCpointer,DCbool,s_lslfpclpififfpB)
VF5(11,DCchar,DCfloat,DCshort,DCdouble,DCpointer,s_cfsdp)
VF10(12,DCbool,DCint,DCdouble,DClonglong,DCint,DCbool,DCint,DCpointer,DCchar,DCdouble,s_BidliBipcd)
VF8(13,DCpointer,DCbool,DCint,DCpointer,DCchar,DCdouble,DClonglong,DCint,s_pBipcdli)
VF18(14,DCfloat,DCshort,DCint,DCpointer,DCdouble,DCchar,DClonglong,DCbool,DCint,DCfloat,DCshort,DCdouble,DCbool,DCbool,DCpointer,DClonglong,DClonglong,DCdouble,s_fsipdclBifsdBBplld)
VF12(15,DCfloat,DCdouble,DClonglong,DCpointer,DCint,DCchar,DCint,DCbool,DCbool,DClonglong,DClonglong,DCpointer,s_fdlpiciBBllp)
VF13(16,DCfloat,DCpointer,DCchar,DCpointer,DCbool,DCshort,DCpointer,DCfloat,DCpointer,DClonglong,DCchar,DCbool,DCchar,s_fpcpBspfplcBc)
VF17(17,DCfloat,DCshort,DCshort,DCfloat,DCchar,DCfloat,DCdouble,DCfloat,DCshort,DCfloat,DCbool,DCfloat,DCchar,DCfloat,DCchar,DClonglong,DCfloat,s_fssfcfdfsfBfcfclf)
VF14(18,DCchar,DCdouble,DCbool,DCdouble,DCfloat,DCint,DCbool,DCfloat,DCchar,DCshort,DCpointer,DCchar,DCint,DCint,s_cdBdfiBfcspcii)
VF11(19,DCshort,DCfloat,DCchar,DClonglong,DCfloat,DCfloat,DCchar,DClonglong,DClonglong,DCchar,DCpointer,s_sfclffcllcp)
VF14(20,DCint,DCshort,DCchar,DCchar,DCbool,DCpointer,DCfloat,DCdouble,DCbool,DCbool,DCfloat,DCint,DCfloat,DCpointer,s_isccBpfdBBfifp)
VF9(21,DCdouble,DCshort,DCdouble,DCbool,DCshort,DClonglong,DCshort,DCfloat,DCshort,s_dsdBslsfs)
VF3(22,DCbool,DCint,DCchar,s_Bic)
VF18(23,DCchar,DCshort,DCchar,DCchar,DCdouble,DCshort,DCchar,DCchar,DCfloat,DCint,DCbool,DClonglong,DCfloat,DCint,DCpointer,DCchar,DClonglong,DCchar,s_csccdsccfiBlfipclc)
VF12(24,DCchar,DCfloat,DCshort,DCshort,DCdouble,DCbool,DCchar,DClonglong,DClonglong,DCfloat,DCbool,DCchar,s_cfssdBcllfBc)
VF13(25,DCfloat,DClonglong,DCint,DCbool,DCdouble,DCfloat,DCint,DCshort,DCbool,DCchar,DCdouble,DCshort,DCbool,s_fliBdfisBcdsB)
VF6(26,DCchar,DCbool,DCshort,DCchar,DClonglong,DCbool,s_cBsclB)
VF5(27,DCchar,DCint,DCshort,DCchar,DCshort,s_ciscs)
VF9(28,DCbool,DCint,DCint,DCbool,DCbool,DCshort,DCfloat,DCfloat,DClonglong,s_BiiBBsffl)
VF12(29,DCpointer,DCfloat,DCshort,DCpointer,DCdouble,DCdouble,DCshort,DCint,DClonglong,DCchar,DCshort,DCint,s_pfspddsilcsi)
VF1(30,DCpointer,s_p)
VF6(31,DCpointer,DCbool,DCbool,DCfloat,DClonglong,DCpointer,s_pBBflp)
VF13(32,DCchar,DCint,DCdouble,DCchar,DCdouble,DCchar,DCpointer,DCdouble,DCbool,DCint,DCfloat,DCpointer,DCint,s_cidcdcpdBifpi)
VF11(33,DCpointer,DCfloat,DCshort,DCint,DCfloat,DClonglong,DCpointer,DCint,DCshort,DCchar,DCshort,s_pfsiflpiscs)
VF3(34,DCchar,DCchar,DCchar,s_ccc)
VF17(35,DCpointer,DCshort,DCchar,DCint,DCchar,DCpointer,DCchar,DCshort,DCshort,DCbool,DCbool,DCshort,DCdouble,DCfloat,DCchar,DCfloat,DCdouble,s_pscicpcssBBsdfcfd)
VF6(36,DCchar,DCchar,DCshort,DCpointer,DCfloat,DCchar,s_ccspfc)
VF4(37,DClonglong,DCchar,DCfloat,DClonglong,s_lcfl)
VF2(38,DCdouble,DClonglong,s_dl)
VF1(39,DCint,s_i)
VF8(40,DCpointer,DCshort,DCfloat,DCfloat,DCchar,DCbool,DClonglong,DCdouble,s_psffcBld)
VF1(41,DCdouble,s_d)
VF14(42,DCpointer,DCshort,DCdouble,DCchar,DCshort,DCbool,DCdouble,DCshort,DCchar,DCbool,DCdouble,DCpointer,DCchar,DCbool,s_psdcsBdscBdpcB)
VF10(43,DCshort,DClonglong,DClonglong,DClonglong,DCshort,DCshort,DCpointer,DCdouble,DCbool,DCbool,s_slllsspdBB)
VF2(44,DCint,DCchar,s_ic)
VF4(45,DCint,DCfloat,DClonglong,DCchar,s_iflc)
VF18(46,DCbool,DCshort,DCint,DCfloat,DClonglong,DCdouble,DCbool,DCfloat,DCpointer,DCdouble,DCpointer,DCpointer,DCbool,DCfloat,DCshort,DCbool,DCbool,DClonglong,s_BsifldBfpdppBfsBBl)
VF10(47,DCpointer,DCpointer,DCfloat,DCdouble,DCfloat,DClonglong,DCfloat,DCshort,DCint,DCshort,s_ppfdflfsis)
VF12(48,DCint,DCint,DCbool,DCint,DCdouble,DCbool,DCbool,DCdouble,DClonglong,DCchar,DCdouble,DCchar,s_iiBidBBdlcdc)
VF2(49,DCdouble,DCint,s_di)
VF1(50,DCdouble,s_d)
VF11(51,DCint,DCfloat,DCdouble,DCbool,DCshort,DCchar,DCdouble,DCint,DCchar,DClonglong,DCdouble,s_ifdBscdicld)
VF11(52,DCpointer,DCchar,DCdouble,DCfloat,DCdouble,DClonglong,DCpointer,DCfloat,DCint,DCbool,DCchar,s_pcdfdlpfiBc)
VF10(53,DCshort,DCfloat,DCpointer,DCfloat,DCshort,DCdouble,DCbool,DClonglong,DCshort,DClonglong,s_sfpfsdBlsl)
VF12(54,DCfloat,DCbool,DCfloat,DCchar,DCchar,DCpointer,DCshort,DCpointer,DCbool,DCbool,DCfloat,DClonglong,s_fBfccpspBBfl)
VF7(55,DCint,DCchar,DCchar,DCint,DCbool,DCfloat,DCbool,s_icciBfB)
VF19(56,DClonglong,DClonglong,DCfloat,DCbool,DCchar,DClonglong,DCchar,DClonglong,DCint,DClonglong,DCdouble,DCchar,DCpointer,DCfloat,DCdouble,DCpointer,DCdouble,DCdouble,DCfloat,s_llfBclclildcpfdpddf)
VF12(57,DCint,DCfloat,DCbool,DCshort,DCpointer,DCshort,DCbool,DCfloat,DCpointer,DCchar,DCpointer,DCdouble,s_ifBspsBfpcpd)
VF17(58,DCchar,DCchar,DCfloat,DCshort,DCshort,DCshort,DCbool,DCint,DCbool,DCbool,DCshort,DCpointer,DCpointer,DCfloat,DClonglong,DCint,DClonglong,s_ccfsssBiBBsppflil)
VF1(59,DCpointer,s_p)
VF17(60,DCdouble,DClonglong,DCfloat,DClonglong,DCdouble,DClonglong,DCbool,DCchar,DCfloat,DCbool,DCbool,DClonglong,DCpointer,DCfloat,DClonglong,DCint,DCpointer,s_dlfldlBcfBBlpflip)
VF4(61,DCshort,DCdouble,DCint,DCint,s_sdii)
VF4(62,DCint,DCshort,DCfloat,DClonglong,s_isfl)
VF4(63,DCint,DCpointer,DClonglong,DCshort,s_ipls)
VF1(64,DCdouble,s_d)
VF17(65,DCdouble,DCchar,DCchar,DClonglong,DCpointer,DCshort,DClonglong,DCdouble,DCfloat,DCshort,DClonglong,DCdouble,DCbool,DCshort,DCfloat,DCchar,DClonglong,s_dcclpsldfsldBsfcl)
VF2(66,DClonglong,DClonglong,s_ll)
VF4(67,DCpointer,DCfloat,DCfloat,DCint,s_pffi)
VF14(68,DCfloat,DCdouble,DCpointer,DCbool,DCbool,DCpointer,DCpointer,DCshort,DCchar,DCpointer,DCpointer,DCfloat,DCfloat,DCbool,s_fdpBBppscppffB)
VF4(69,DCchar,DCint,DCshort,DClonglong,s_cisl)
VF3(70,DCbool,DCint,DCpointer,s_Bip)
VF1(71,DCint,s_i)
VF4(72,DCchar,DCbool,DClonglong,DCfloat,s_cBlf)
VF13(73,DCdouble,DCchar,DCshort,DCfloat,DCshort,DCchar,DCbool,DCdouble,DCpointer,DClonglong,DCdouble,DCfloat,DCbool,s_dcsfscBdpldfB)
VF17(74,DCfloat,DClonglong,DCbool,DCbool,DClonglong,DCchar,DClonglong,DCbool,DCfloat,DClonglong,DClonglong,DCshort,DCshort,DCshort,DCpointer,DCchar,DCchar,s_flBBlclBfllssspcc)
VF16(75,DCshort,DClonglong,DClonglong,DCdouble,DCshort,DCint,DCchar,DCshort,DCint,DClonglong,DCbool,DCdouble,DCbool,DCdouble,DCfloat,DCbool,s_slldsicsilBdBdfB)
VF6(76,DClonglong,DClonglong,DCshort,DCpointer,DCfloat,DCchar,s_llspfc)
VF9(77,DCfloat,DCpointer,DCshort,DClonglong,DCdouble,DCshort,DCshort,DCfloat,DCbool,s_fpsldssfB)
VF12(78,DCbool,DCbool,DCfloat,DCpointer,DCshort,DCchar,DCfloat,DCint,DCdouble,DCchar,DCfloat,DCdouble,s_BBfpscfidcfd)
VF18(79,DCint,DCint,DCint,DCshort,DCdouble,DCchar,DCbool,DCbool,DCchar,DClonglong,DCfloat,DCdouble,DCpointer,DCdouble,DCbool,DCint,DCpointer,DClonglong,s_iiisdcBBclfdpdBipl)
VF12(80,DCpointer,DCchar,DCbool,DClonglong,DCchar,DCfloat,DCpointer,DCbool,DCchar,DCchar,DCshort,DCdouble,s_pcBlcfpBccsd)
VF19(81,DCchar,DCshort,DCint,DCdouble,DCbool,DCshort,DCfloat,DCbool,DCfloat,DCchar,DCfloat,DCpointer,DCfloat,DCfloat,DCpointer,DCshort,DCchar,DCdouble,DCint,s_csidBsfBfcfpffpscdi)
VF14(82,DCint,DCbool,DCchar,DCfloat,DCbool,DClonglong,DCshort,DCfloat,DCfloat,DCchar,DCshort,DCbool,DCbool,DCchar,s_iBcfBlsffcsBBc)
VF8(83,DCpointer,DCchar,DCint,DClonglong,DClonglong,DCint,DCchar,DCfloat,s_pcillicf)
VF17(84,DCdouble,DCshort,DCchar,DClonglong,DCshort,DCdouble,DCshort,DCfloat,DCdouble,DCshort,DCdouble,DCdouble,DClonglong,DCbool,DCdouble,DCint,DCpointer,s_dsclsdsfdsddlBdip)
VF17(85,DCdouble,DCpointer,DClonglong,DCpointer,DCfloat,DCshort,DCpointer,DCshort,DCshort,DCdouble,DCint,DCpointer,DCshort,DCshort,DCint,DCpointer,DClonglong,s_dplpfspssdipssipl)
VF3(86,DCbool,DCbool,DCbool,s_BBB)
VF18(87,DCpointer,DClonglong,DClonglong,DCfloat,DClonglong,DCpointer,DCfloat,DCdouble,DClonglong,DCpointer,DCfloat,DClonglong,DClonglong,DCbool,DCint,DCpointer,DCpointer,DCint,s_pllflpfdlpfllBippi)
VF5(88,DCint,DCfloat,DCfloat,DCdouble,DCfloat,s_iffdf)
VF13(89,DCdouble,DCshort,DClonglong,DCchar,DCchar,DCfloat,DCshort,DCbool,DCfloat,DCbool,DCchar,DCint,DCpointer,s_dslccfsBfBcip)
VF6(90,DCshort,DCfloat,DCchar,DCbool,DCpointer,DCshort,s_sfcBps)
VF5(91,DCshort,DCchar,DCshort,DCfloat,DCint,s_scsfi)
VF8(92,DCshort,DClonglong,DCdouble,DCfloat,DCdouble,DCchar,DClonglong,DCdouble,s_sldfdcld)
VF3(93,DCfloat,DCdouble,DCshort,s_fds)
VF15(94,DCint,DCfloat,DCshort,DCshort,DCbool,DCint,DCint,DCpointer,DCpointer,DCshort,DCint,DCfloat,DCshort,DCshort,DCfloat,s_ifssBiippsifssf)
VF6(95,DCchar,DCbool,DClonglong,DCint,DClonglong,DCpointer,s_cBlilp)
VF8(96,DCbool,DClonglong,DCchar,DCfloat,DCbool,DCfloat,DCbool,DCdouble,s_BlcfBfBd)
