/*

 Package: dyncall
 Library: test
 File: test/suite2_x86win32std/case.h
 Description: 
 License:

   Copyright (c) 2011-2018 Daniel Adler <dadler@uni-goettingen.de>,
                           Tassilo Philipp <tphilipp@potion-studios.com>

   Permission to use, copy, modify, and distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
   copyright notice and this permission notice appear in all copies.

   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
   WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
   ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

*/

/* auto generated by mkcase.py (on stdout) */
VF16(0,DClonglong,DCbool,DCshort,DCint,DCint,DCfloat,DCdouble,DCpointer,DCdouble,DClonglong,DCshort,DCpointer,DClonglong,DCdouble,DCint,DCbool,s_lBsiifdpdlspldiB)
VF19(1,DCdouble,DCint,DCshort,DCchar,DCshort,DCbool,DClonglong,DCbool,DCbool,DCpointer,DCint,DCpointer,DCint,DCbool,DCshort,DClonglong,DCbool,DCchar,DCpointer,s_discsBlBBpipiBslBcp)
VF12(2,DClonglong,DCshort,DCdouble,DClonglong,DCfloat,DCdouble,DCpointer,DCfloat,DClonglong,DCpointer,DCint,DCint,s_lsdlfdpflpii)
VF5(3,DCdouble,DCdouble,DCint,DCshort,DCshort,s_ddiss)
VF8(4,DClonglong,DCfloat,DCbool,DCshort,DCdouble,DCdouble,DClonglong,DCchar,s_lfBsddlc)
VF10(5,DCshort,DCint,DCshort,DCbool,DCdouble,DCbool,DCint,DCbool,DCfloat,DCshort,s_sisBdBiBfs)
VF9(6,DCshort,DCbool,DCbool,DClonglong,DCchar,DCfloat,DCpointer,DCshort,DCint,s_sBBlcfpsi)
VF17(7,DCfloat,DCchar,DCpointer,DCint,DCchar,DCdouble,DCdouble,DCpointer,DCshort,DCbool,DCdouble,DCint,DCbool,DCpointer,DCfloat,DCchar,DCshort,s_fcpicddpsBdiBpfcs)
VF7(8,DCchar,DCbool,DCfloat,DCchar,DCshort,DClonglong,DCpointer,s_cBfcslp)
VF13(9,DCint,DCfloat,DCint,DCchar,DCbool,DCint,DClonglong,DClonglong,DCchar,DCpointer,DCpointer,DCshort,DCpointer,s_ificBillcppsp)
VF15(10,DCpointer,DCdouble,DCpointer,DClonglong,DCint,DCshort,DCfloat,DCbool,DCpointer,DCpointer,DCint,DCpointer,DCdouble,DCfloat,DCdouble,s_pdplisfBppipdfd)
VF4(11,DClonglong,DCshort,DCshort,DCint,s_lssi)
VF4(12,DClonglong,DCint,DClonglong,DCchar,s_lilc)
VF17(13,DClonglong,DCbool,DCdouble,DCdouble,DCdouble,DCpointer,DCpointer,DCfloat,DCchar,DCbool,DClonglong,DCshort,DClonglong,DCdouble,DCdouble,DClonglong,DCpointer,s_lBdddppfcBlslddlp)
VF2(14,DCbool,DCchar,s_Bc)
VF18(15,DCchar,DCbool,DCint,DCshort,DCdouble,DCint,DCchar,DClonglong,DCpointer,DCbool,DCdouble,DCint,DCshort,DClonglong,DCpointer,DCpointer,DCpointer,DCchar,s_cBisdiclpBdislpppc)
VF17(16,DCbool,DCdouble,DClonglong,DCdouble,DCpointer,DClonglong,DCdouble,DCdouble,DClonglong,DCshort,DCfloat,DCfloat,DCchar,DCbool,DCpointer,DCshort,DCdouble,s_BdldplddlsffcBpsd)
VF8(17,DCfloat,DClonglong,DCchar,DCfloat,DClonglong,DCint,DClonglong,DCfloat,s_flcflilf)
VF12(18,DCpointer,DCshort,DCshort,DClonglong,DCshort,DCpointer,DCbool,DCpointer,DCshort,DCchar,DCbool,DClonglong,s_psslspBpscBl)
VF7(19,DCchar,DCchar,DClonglong,DCint,DCpointer,DCshort,DCdouble,s_cclipsd)
VF15(20,DCshort,DCfloat,DCbool,DCdouble,DCdouble,DCint,DCshort,DCchar,DCpointer,DCbool,DClonglong,DCshort,DCint,DCbool,DCbool,s_sfBddiscpBlsiBB)
VF4(21,DCbool,DClonglong,DCchar,DCdouble,s_Blcd)
VF19(22,DCchar,DCfloat,DCint,DCchar,DCfloat,DClonglong,DClonglong,DClonglong,DCchar,DCchar,DCbool,DCbool,DCshort,DClonglong,DCbool,DCfloat,DCchar,DCchar,DClonglong,s_cficflllccBBslBfccl)
VF14(23,DClonglong,DCpointer,DCbool,DCdouble,DCint,DCint,DCfloat,DClonglong,DCfloat,DCint,DClonglong,DCint,DCint,DCfloat,s_lpBdiiflfiliif)
VF16(24,DCshort,DCbool,DCbool,DCpointer,DCint,DCshort,DClonglong,DCfloat,DCint,DCbool,DCbool,DCshort,DCpointer,DCfloat,DCbool,DCdouble,s_sBBpislfiBBspfBd)
VF19(25,DClonglong,DClonglong,DClonglong,DCdouble,DClonglong,DCbool,DCpointer,DCint,DClonglong,DCbool,DCshort,DCpointer,DCint,DCint,DCfloat,DCfloat,DCshort,DCint,DCshort,s_llldlBpilBspiiffsis)
VF12(26,DCdouble,DCint,DCpointer,DCshort,DCpointer,DCchar,DCfloat,DCint,DCint,DCchar,DCdouble,DClonglong,s_dipspcfiicdl)
VF15(27,DCfloat,DCfloat,DCpointer,DCbool,DCshort,DClonglong,DCbool,DCbool,DCchar,DCshort,DClonglong,DCchar,DCbool,DCfloat,DCbool,s_ffpBslBBcslcBfB)
VF2(28,DCfloat,DCchar,s_fc)
VF9(29,DCdouble,DCfloat,DCfloat,DCint,DCpointer,DCbool,DCfloat,DCpointer,DCchar,s_dffipBfpc)
VF10(30,DClonglong,DClonglong,DCfloat,DCshort,DCint,DClonglong,DCint,DCdouble,DCdouble,DCbool,s_llfsiliddB)
VF11(31,DCfloat,DCshort,DCbool,DClonglong,DCint,DCdouble,DCint,DClonglong,DCfloat,DCshort,DCshort,s_fsBlidilfss)
VF11(32,DCchar,DCint,DCfloat,DClonglong,DCbool,DCdouble,DCpointer,DCshort,DClonglong,DCchar,DCfloat,s_ciflBdpslcf)
VF2(33,DCint,DClonglong,s_il)
VF3(34,DClonglong,DCshort,DCchar,s_lsc)
VF5(35,DCfloat,DCchar,DCfloat,DCchar,DCfloat,s_fcfcf)
VF3(36,DCdouble,DCint,DCfloat,s_dif)
VF13(37,DCint,DCfloat,DCbool,DClonglong,DCfloat,DClonglong,DCchar,DCpointer,DCpointer,DCchar,DCshort,DCfloat,DCint,s_ifBlflcppcsfi)
VF2(38,DCbool,DCint,s_Bi)
VF6(39,DCpointer,DCchar,DCdouble,DCshort,DCfloat,DCint,s_pcdsfi)
VF5(40,DCshort,DClonglong,DCpointer,DCbool,DCchar,s_slpBc)
VF4(41,DCdouble,DCbool,DCdouble,DCfloat,s_dBdf)
VF16(42,DCshort,DCchar,DClonglong,DCpointer,DCint,DClonglong,DClonglong,DClonglong,DCint,DCdouble,DCchar,DCpointer,DCshort,DCbool,DCchar,DCint,s_sclpilllidcpsBci)
VF15(43,DCshort,DCfloat,DCint,DCshort,DCfloat,DCbool,DClonglong,DClonglong,DCpointer,DCbool,DCdouble,DCint,DCpointer,DCfloat,DCshort,s_sfisfBllpBdipfs)
VF2(44,DCfloat,DCdouble,s_fd)
VF6(45,DClonglong,DCint,DCpointer,DCfloat,DCpointer,DCshort,s_lipfps)
VF8(46,DCchar,DCshort,DCdouble,DCfloat,DCdouble,DCbool,DCchar,DCfloat,s_csdfdBcf)
VF10(47,DCshort,DCint,DCpointer,DCfloat,DCchar,DCshort,DCint,DCdouble,DCpointer,DCdouble,s_sipfcsidpd)
VF16(48,DCshort,DCdouble,DCpointer,DCbool,DCbool,DCbool,DCbool,DCbool,DCint,DClonglong,DCshort,DCint,DCbool,DCfloat,DCint,DClonglong,s_sdpBBBBBilsiBfil)
VF3(49,DCchar,DClonglong,DCchar,s_clc)
VF9(50,DCbool,DCpointer,DCfloat,DClonglong,DCbool,DCfloat,DClonglong,DCbool,DCpointer,s_BpflBflBp)
VF17(51,DClonglong,DCshort,DCfloat,DCchar,DClonglong,DCdouble,DCpointer,DClonglong,DCpointer,DCint,DCfloat,DCint,DCint,DCint,DCchar,DCpointer,DCdouble,s_lsfcldplpifiiicpd)
VF4(52,DCint,DCchar,DCint,DCdouble,s_icid)
VF16(53,DCshort,DCint,DCbool,DClonglong,DCint,DCshort,DCbool,DClonglong,DCfloat,DCbool,DCpointer,DCint,DCint,DCpointer,DCdouble,DCchar,s_siBlisBlfBpiipdc)
VF5(54,DCpointer,DCint,DClonglong,DCshort,DCfloat,s_pilsf)
VF5(55,DCbool,DCint,DCshort,DCdouble,DCshort,s_Bisds)
VF18(56,DCbool,DCdouble,DCchar,DCint,DCdouble,DCbool,DCpointer,DCint,DClonglong,DCbool,DClonglong,DCshort,DCshort,DCdouble,DCshort,DCbool,DCint,DCbool,s_BdcidBpilBlssdsBiB)
VF6(57,DCshort,DCbool,DCshort,DCshort,DCdouble,DClonglong,s_sBssdl)
VF8(58,DCchar,DCfloat,DCbool,DCchar,DCint,DCint,DCshort,DCfloat,s_cfBciisf)
VF3(59,DCshort,DCdouble,DClonglong,s_sdl)
VF16(60,DCshort,DCint,DCfloat,DClonglong,DCbool,DCdouble,DCint,DCbool,DCchar,DCpointer,DCpointer,DClonglong,DCchar,DCshort,DCbool,DCfloat,s_siflBdiBcpplcsBf)
VF7(61,DCdouble,DCpointer,DCint,DCdouble,DCbool,DCchar,DCdouble,s_dpidBcd)
VF18(62,DCpointer,DCbool,DCint,DCchar,DCdouble,DCfloat,DCfloat,DCfloat,DClonglong,DCfloat,DClonglong,DCfloat,DCpointer,DCchar,DClonglong,DCpointer,DCbool,DCfloat,s_pBicdffflflfpclpBf)
VF7(63,DCshort,DCbool,DCchar,DCdouble,DCfloat,DCbool,DCpointer,s_sBcdfBp)
VF8(64,DCchar,DCchar,DCfloat,DCpointer,DCpointer,DClonglong,DCshort,DClonglong,s_ccfpplsl)
VF9(65,DClonglong,DCdouble,DCdouble,DCdouble,DCbool,DCpointer,DCbool,DCfloat,DCchar,s_ldddBpBfc)
VF12(66,DCbool,DCdouble,DCpointer,DCdouble,DCpointer,DCpointer,DCint,DCshort,DCchar,DClonglong,DCpointer,DClonglong,s_Bdpdppisclpl)
VF12(67,DCshort,DClonglong,DCshort,DClonglong,DCchar,DCint,DCpointer,DCpointer,DCshort,DCint,DClonglong,DCchar,s_slslcippsilc)
VF3(68,DCshort,DCbool,DCfloat,s_sBf)
VF12(69,DCdouble,DCpointer,DCpointer,DCfloat,DClonglong,DClonglong,DCint,DCchar,DCpointer,DCchar,DCdouble,DCbool,s_dppfllicpcdB)
VF18(70,DCshort,DCshort,DCchar,DCfloat,DCint,DCdouble,DCchar,DCchar,DCint,DCshort,DCfloat,DClonglong,DCfloat,DCshort,DCdouble,DCpointer,DCshort,DClonglong,s_sscfidccisflfsdpsl)
VF16(71,DClonglong,DCshort,DCbool,DCchar,DClonglong,DCshort,DCdouble,DCbool,DCchar,DCshort,DCchar,DCdouble,DClonglong,DCfloat,DCdouble,DCbool,s_lsBclsdBcscdlfdB)
VF11(72,DCshort,DCchar,DCfloat,DCfloat,DClonglong,DCpointer,DCfloat,DCfloat,DCpointer,DCshort,DCdouble,s_scfflpffpsd)
VF14(73,DClonglong,DCchar,DCint,DCbool,DCdouble,DCbool,DCdouble,DCdouble,DCbool,DCfloat,DCchar,DCbool,DCdouble,DCbool,s_lciBdBddBfcBdB)
VF4(74,DCshort,DClonglong,DCbool,DClonglong,s_slBl)
VF12(75,DCbool,DCbool,DCint,DCdouble,DCchar,DCpointer,DCbool,DCfloat,DCshort,DCbool,DCbool,DCchar,s_BBidcpBfsBBc)
VF5(76,DCpointer,DClonglong,DCint,DCshort,DCchar,s_plisc)
VF1(77,DCchar,s_c)
VF15(78,DCpointer,DCdouble,DCfloat,DCbool,DCint,DCint,DCfloat,DCfloat,DCpointer,DCfloat,DCchar,DCdouble,DCfloat,DCshort,DCshort,s_pdfBiiffpfcdfss)
VF14(79,DCdouble,DCchar,DCchar,DCshort,DCfloat,DClonglong,DClonglong,DCchar,DCdouble,DCbool,DCfloat,DCfloat,DClonglong,DCchar,s_dccsfllcdBfflc)
VF18(80,DCpointer,DCshort,DCfloat,DCbool,DClonglong,DCshort,DCint,DCint,DCdouble,DCbool,DCshort,DCpointer,DCpointer,DCchar,DCbool,DCint,DCfloat,DCdouble,s_psfBlsiidBsppcBifd)
VF14(81,DClonglong,DClonglong,DCint,DCdouble,DClonglong,DClonglong,DCbool,DClonglong,DClonglong,DClonglong,DCshort,DClonglong,DCdouble,DClonglong,s_llidllBlllsldl)
VF8(82,DCchar,DCshort,DCbool,DCfloat,DCshort,DCfloat,DCshort,DClonglong,s_csBfsfsl)
VF12(83,DCfloat,DCbool,DCint,DCshort,DCpointer,DCdouble,DCshort,DCpointer,DClonglong,DCbool,DCpointer,DClonglong,s_fBispdsplBpl)
VF7(84,DCpointer,DCbool,DCchar,DCfloat,DClonglong,DCdouble,DCfloat,s_pBcfldf)
VF18(85,DCshort,DClonglong,DCbool,DCbool,DCpointer,DCdouble,DCint,DCdouble,DCshort,DCchar,DCfloat,DCpointer,DCchar,DCint,DCint,DClonglong,DCint,DCpointer,s_slBBpdidscfpciilip)
VF2(86,DCpointer,DCshort,s_ps)
VF17(87,DCbool,DCint,DCint,DCchar,DCchar,DCchar,DCchar,DCbool,DCint,DCchar,DCbool,DCdouble,DCint,DCfloat,DCchar,DCpointer,DCshort,s_BiiccccBicBdifcps)
VF7(88,DCpointer,DCshort,DCdouble,DCchar,DCpointer,DCpointer,DCshort,s_psdcpps)
VF14(89,DCpointer,DCbool,DCbool,DCfloat,DCbool,DCpointer,DCint,DCdouble,DCchar,DClonglong,DCdouble,DClonglong,DCfloat,DCbool,s_pBBfBpidcldlfB)
VF17(90,DCchar,DCchar,DCint,DCshort,DCfloat,DCchar,DCint,DCdouble,DCdouble,DCdouble,DCint,DClonglong,DCdouble,DCshort,DCbool,DCint,DCbool,s_ccisfcidddildsBiB)
VF4(91,DCbool,DCint,DCshort,DCpointer,s_Bisp)
VF11(92,DCdouble,DCshort,DCint,DCint,DCshort,DCpointer,DCchar,DCint,DCbool,DCdouble,DClonglong,s_dsiispciBdl)
VF16(93,DCbool,DCpointer,DCshort,DClonglong,DCshort,DCchar,DClonglong,DCshort,DCshort,DCint,DCint,DCchar,DClonglong,DCdouble,DCchar,DCdouble,s_Bpslsclssiicldcd)
